require 'singleton'
require 'brokers/main_broker'
require 'chili_logger/version'
require 'current_log_accessor'
require 'errors/logging_error_handler/logging_error_handler'
require 'helpers/values/default'
require 'helpers/values/type_uniformizer/desc'
require 'helpers/values/type_uniformizer/user'
require 'helpers/values/type_uniformizer/main_content'
require 'message_writer/message_writer'
require 'unpublished_logs_manager/unpublished_logs_manager'

# class for centralizing Creative's logging logic
class ChiliLogger
  attr_reader :msg_broker
  attr_accessor :deactivated
  # this class uses the singleton module, so instead of initializing it with ChiliLogger.new,
  # initialize it with ChiliLogger.instance
  include Singleton

  def initialize
    @default = ChiliLogger::Values::Default.new
    @user_uniformizer = ChiliLogger::Values::TypeUniformizer::User.new
    @desc_uniformizer = ChiliLogger::Values::TypeUniformizer::Desc.new
    @main_content_uniformizer = ChiliLogger::Values::TypeUniformizer::MainContent.new
  end

  def config(config)
    @deactivated = config[:deactivated] || false # ChilLogger can be deactivated for test environents

    config_logging_error_handler(config)
    config_msg_writer(config)
    config_msg_broker(config)
    config_unpublished_logs_manager

    @current_log_accessor = CurrentLogAccessor.new(@msg_broker, @msg_writer)
  end

  def current_log
    @current_log_accessor
  end

  def start_new_log(**options)
    current_log.clear_log_info

    current_log.update_desc(options[:desc])
    current_log.update_user(options[:user])
    current_log.update_main_content(options[:main_content])

    current_log
  end

  def publish_instant_log(**options)
    # enforces that all attributes have valid primitive types
    if options
      options[:desc] = @desc_uniformizer.desc(options[:desc])
      options[:user] = @user_uniformizer.user(options[:user])
      options[:main_content] = @main_content_uniformizer.main_content(options[:main_content])
    end

    message = @msg_writer.write(options)
    @msg_broker.publish(message)
  end

  def republish_unpublished_logs
    @unpublished_logs_manager.republish
  end

  private

  def config_logging_error_handler(general_config)
    fallback_name = general_config[:fallback_broker]
    fallback_config = general_config[:fallback_broker_config]
    @logging_error_handler = LoggingErrorHandler.new(fallback_name, fallback_config)
  end

  def config_msg_writer(general_config)
    msg_writer_config = {
      env: general_config[:log_env]&.downcase,
      layer: general_config[:log_layer]&.downcase,
      server_url: general_config[:server_url],
      cloud_provider: general_config[:cloud_provider]
    }

    @msg_writer = MessageWriter.new(msg_writer_config)
  end

  def config_msg_broker(general_config)
    msg_broker_name = general_config[:msg_broker_name]
    msg_broker_config = general_config[:msg_broker_config]

    @msg_broker = MainBroker.new(msg_broker_name, msg_broker_config, @logging_error_handler)
  end

  def config_unpublished_logs_manager
    @unpublished_logs_manager = UnpublishedLogsManager.new(@msg_broker, @logging_error_handler)
  end
end
