require 'message_writer/message_writer'
require 'helpers/values/type_uniformizer/desc'
require 'helpers/values/type_uniformizer/main_content'
require 'helpers/values/type_uniformizer/user'

# class for centralizing Creative's logging logic
class ChiliLogger
  # class for manipulating current thread variables
  class CurrentLogAccessor
    def initialize(msg_broker, msg_writer)
      @msg_broker = msg_broker
      @msg_writer = msg_writer
      @default = ChiliLogger::Values::Default.new
      @user_uniformizer = ChiliLogger::Values::TypeUniformizer::User.new
      @desc_uniformizer = ChiliLogger::Values::TypeUniformizer::Desc.new
      @main_content_uniformizer = ChiliLogger::Values::TypeUniformizer::MainContent.new
    end

    def publish(**options)
      update_current_log(options)
      message = @msg_writer.write(desc: desc, user: user, main_content: main_content)
      @msg_broker.publish(message)

      clear_log_info
    end

    # ACCESSOR METHODS BELOW
    # current log infos are stored in thread variables, so they can be accessed anywhere(controllers, models, tasks...)
    def user
      Thread.current[:current_log_user] ||= @default.user
    end

    def update_user(new_user)
      return Thread.current[:current_log_user] = nil if new_user.nil?

      uniformized_user = @user_uniformizer.user(new_user)
      Thread.current[:current_log_user] = user.merge(uniformized_user)
    end

    def desc
      Thread.current[:current_log_desc] ||= @default.desc
    end

    def update_type(new_type_name)
      desc[:type] = @desc_uniformizer.type(new_type_name)
    end

    def update_service(new_service_name)
      desc[:service] = @desc_uniformizer.service(new_service_name)
    end

    def update_action(new_action_name)
      desc[:action] = @desc_uniformizer.action(new_action_name)
    end

    def update_desc(new_desc)
      return Thread.current[:current_log_desc] = nil if new_desc.nil?

      uniformized_desc = @desc_uniformizer.desc(new_desc)
      Thread.current[:current_log_desc] = desc.merge(uniformized_desc)
    end

    def main_content
      Thread.current[:current_log_main_content] ||= @default.main_content
    end

    def update_main_content(new_content)
      return Thread.current[:current_log_main_content] = nil if new_content.nil?

      uniform_content = @main_content_uniformizer.main_content(new_content)
      uniform_recs = @main_content_uniformizer.modified_records(uniform_content[:modified_records])
      uniform_errors = @main_content_uniformizer.errors(uniform_content[:errors])

      # only overwrite if key was explicitly set
      main_content[:modified_records] = uniform_recs if uniform_content.key?(:modified_records)
      main_content[:errors] = uniform_errors if uniform_content.key?(:errors)
    end

    def add_to_main_content(new_content)
      uniform_content = @main_content_uniformizer.main_content(new_content)
      uniform_recs = uniform_content[:modified_records]
      uniform_errs = uniform_content[:errors]

      # only overwrite if key was explicitly set
      main_content[:modified_records] = modified_records.merge(uniform_recs) if uniform_recs
      main_content[:errors] = errors + uniform_errs if uniform_errs
    end

    def modified_records
      main_content[:modified_records] ||= @default.modified_records
    end

    def overwrite_modified_records(new_mod_recs)
      return main_content[:modified_records] = nil if new_mod_recs.nil?

      uniformized_recs = @main_content_uniformizer.modified_records(new_mod_recs)
      main_content[:modified_records] = uniformized_recs
    end

    def add_modified_record(table_name, new_mod_rec)
      return unless table_name.is_a?(String)

      uniformized_rec = @main_content_uniformizer.modified_record(new_mod_rec)
      modified_records[table_name] ||= []
      modified_records[table_name] = modified_records[table_name].push(uniformized_rec)
    end

    def errors
      main_content[:errors] ||= @default.log_errors
    end

    def overwrite_errors(new_errors)
      return main_content[:errors] = nil if new_errors.nil?

      uniformized_errors = @main_content_uniformizer.errors(new_errors)
      main_content[:errors] = uniformized_errors
    end

    def add_error(error)
      uniformized_error = @main_content_uniformizer.error(error)

      main_content[:errors] = errors.push(uniformized_error)
      errors
    end

    def clear_log_info
      update_desc(nil)
      update_user(nil)
      update_main_content(nil)
    end

    private

    def update_current_log(options)
      update_desc(options[:desc]) if options[:desc]
      update_user(options[:user]) if options[:user]
      update_main_content(options[:main_content]) if options[:main_content]
    end
  end
end
