require 'brokers/sqs_broker'
require 'errors/error_messages/config_error'

# class for centralizing Creative's logging logic
class ChiliLogger
  # class that handles errors when message broker can't be reached, etc...
  class LoggingErrorHandler
    def initialize(fallback_name = nil, config = {})
      return if ChiliLogger.instance.deactivated

      @default = ChiliLogger::Values::Default.new
      config ||= {}
      validate_config(config)

      fallback_name ||= @default.fallback_broker
      fallback_broker_class = supported_fallback_brokers[fallback_name.to_sym]
      unsupported_fallback_broker_error unless fallback_broker_class

      @fallback_broker = fallback_broker_class.new(config)
    end

    def handle_error(error, log = nil)
      return if ChiliLogger.instance.deactivated

      message = message(error, log)
      @fallback_broker.publish(message)
    rescue StandardError => e
      puts 'There was a problem with both the Message Broker and the Fallback Broker simultaneously.
        To keep the application running and prevent downtime,
        ChiliLogger will ignore this errors and discard the log it was currently trying to publish.
        Please note that logs are being permanently lost.'
      puts e
    end

    def fetch_unpublished_logs
      @fallback_broker.fetch_messages
    end

    def delete_unpublished_log(msg)
      @fallback_broker.delete_message(msg)
    end

    private

    def validate_config(config)
      invalid_config_err = ':fallback_broker_config must be a hash'
      raise(ChiliLogger::ConfigError, invalid_config_err) unless [Hash, NilClass].include?(config.class)
    end

    def message(error, log)
      {
        error_type: error.class.name,
        error_message: error.message,
        log: log
      }
    end

    def unsupported_fallback_broker_error
      error_handlers = supported_fallback_brokers.keys.join(', ')
      unsupported_err = ":error_handler must be present and have one of the following values: #{error_handlers}"

      raise(ChiliLogger::ConfigError, unsupported_err)
    end

    def supported_fallback_brokers
      {
        aws_sqs: AWS::SqsBroker
      }
    end
  end
end
